import numpy as np
import matplotlib.pyplot as plt
from glob import glob
from ase.io import read


fig, axes = plt.subplots(2, 3)

#ax = fig.add_subplot(211)
#ax2 = fig.add_subplot(212)

for i, formula in enumerate(['H2O', 'H3N', 'CH4']):
    ax = axes[0, i]
    ax.set_title(formula)
    ax2 = axes[1, i]
    for stencil in [1, 2, 3]:
        thegpts = []
        energies = []
        for gpts in range(32, 72 + 1, 4):
            fname = 'config.{}.stencil{}.gpts{}.traj'.format(formula,
                                                             stencil, gpts)
            try:
                atoms = read(fname)
                e = atoms.get_potential_energy()
                syms = atoms.get_chemical_symbols()
                for i, atom in enumerate(atoms):
                    fname = ('config.{}_{}{}.stencil{}.gpts{}.traj'
                             .format(formula, syms[i], i, stencil, gpts))
                    refatoms = read(fname)
                    eref = refatoms.get_potential_energy()
                    e -= eref
            except Exception:
                print('fail: {}'.format(fname))
            else:
                energies.append(e)
                thegpts.append(gpts)

        ls = '--' if stencil == 3 else '-'
        label = 'stencil={}'.format(stencil)
        hvalues = atoms.cell[0, 0] / np.array(thegpts)
        ax.plot(hvalues, energies, marker='o', ls=ls, label=label)

        diffs = np.array(energies[1:]) - energies[:-1]
        hdiffs = hvalues[:-1] - hvalues[1:]
        diffs /= hdiffs
        ax2.semilogy(hvalues[1:], diffs, marker='o', ls=ls, label=label)


#for a in [axes, ax2]:
axes[0, 0].set_ylabel('Atomization energy')
axes[1, 0].set_ylabel('log dE/dh')
axes[0, 0].legend(loc='lower left')
plt.show()
