import os
import numpy as np
from ase.build import molecule
from ase.io import write
from gpaw import GPAW, Davidson, Mixer
from gpaw.poisson import FFTPoissonSolver as PS

for thing in ['H2O', 'NH3', 'CH4']:
    atoms = molecule(thing)
    atoms.pbc = 1  # For Poisson solver
    atoms.cell = [7., 7., 7.]
    atoms.center()
    #formula = atoms.get_chemical_formula()


    def lockfile(name, comm=None):
        if comm is None:
            from ase.parallel import world
            comm = world

        decision = np.zeros(1, bool)
        if comm.rank == 0:
            lockfilename = 'tmp.{}.lock'.format(name)
            exists = os.path.exists(lockfilename)
            if not exists:
                decision[0] = True
                with open(lockfilename, 'w'):
                    pass
        comm.sum(decision)
        return decision

    def lockfiles(names, comm=None):
        for name in names:
            if lockfile(name, comm):
                yield name

    def calculate(atoms, stencil, gpts, i=None):
        formula = atoms.get_chemical_formula()
        if i is not None:
            atoms = atoms[i:i + 1]
            atoms.set_initial_magnetic_moments([1])
            sym = atoms.get_chemical_symbols()[0]
            formula = '{}_{}{}'.format(formula, sym, i)
        xid = '{}.stencil{}.gpts{}'.format(formula, stencil, gpts)
        if not lockfile(xid):
            return
        calc = GPAW(xc={'name': 'PBE', 'stencil': stencil},
                    gpts=(gpts, gpts, gpts),
                    parallel=dict(kpt=1),
                    poissonsolver=PS(),
                    symmetry='off',
                    eigensolver=Davidson(5),
                    mixer=Mixer(0.6, 5, 20.0),
                    txt='gpaw.{}.txt'.format(xid))
        atoms.calc = calc
        atoms.get_forces()
        write('config.{}.traj'.format(xid), atoms)


    # gpts=36 ~ h=0.2
    for stencil in [1, 2]:
        for gpts in [32, 36, 40, 44, 48, 52, 56, 60, 64, 68, 72]:
            calculate(atoms, stencil, gpts)
            for i in range(len(atoms)):
                calculate(atoms, stencil, gpts, i=i)
